//=============================================================================
// Change Tile Size
// by Shaz
// version 1.2
// Last Update: 2017.04.12
//=============================================================================

/*:
 * @plugindesc Pozwala ma rozmiar kafelków inny, niż domyślne 48x48.
   <ChangeTileSize> v1.1
 * @author Shaz
 *
 * @param Rozmiar tile
 * @desc Rozmiar tile'a (w pikselach)
 * @default 48
 *
 * @param Folder z tilesetami
 * @desc Folder, gdzie są trzymane tilesety używane w grze (nie w edytorze)
 * @default img/tilesets/
 *
 * @param Folder z parallaksami
 * @desc Folder, gdzie są trzymane parallaksy używane w grze (nie w edytorze)
 * @default img/parallaxes/
 *
 * @help Ta wtyczka nie posiada żadnych dodatkowych poleceń.
 *
 * Aby użyć tilesetów o rozmiarze innym niż 48x48 w swoim projekcie, musisz
 * mieć dwa foldery na kafelki. Folder wybrany w ustawieniach wtyczki
 * to oryginalne tilesety, w wybranym rozmiarze. Folder, gdzie normalnie
 * są trzymane kafelki (img/tilesets/) powinien posiadać kopię każdej grafiki
 * tilesetu, ze zmienionym rozmiarem do standardowych 48x48 na kafelek.
 * W edytorze grafiki mogą wyglądać gorzej, ale w grze zostaną użyte te
 * oryginalne. Przed publikacją gry można usunąć zawartość folderu img/
 * tilesets, bo nikomu już nie będzie potrzebna.
 *
 * To samo dotyczy folderu img/parallaxes, jeżeli używasz parallaks do 
 * tzw. parralax mappingu.
 *
 * -- przetłumaczone przez almostnoruby dla rpgmaker.pl
 *
 * ----------------------------------------------------------------------------
 * Wersje:
 *
 * 1.1  2016.10.17 Poprawiono sposób wyświetlania tilesetów aby zapobiec 
 *                 "chropowatości"
 *                 Uniezależniowo wtyczkę od nazwy pliku pluginu
 *      2016.10.23 Wstawiano warunek sprawdzający, czy tilesety/parallaksy
 *                 są dołączone do projekty (aby wyświetlić milszy błąd?)
 * 1.2  2017.04.12 Naprawiono błąd z nieużywanymi zasobami
 * ----------------------------------------------------------------------------
LISTA ZASOBOW - NIE ZMIENIAJ NIC POD NIA
 * @requiredAssets img/tiles_game/TileE
 * @requiredAssets img/tiles_game/TileD
 * @requiredAssets img/tiles_game/TileC
 * @requiredAssets img/tiles_game/TileB
 * @requiredAssets img/tiles_game/TileA5
 * @requiredAssets img/tiles_game/TileA4
 * @requiredAssets img/tiles_game/TileA3
 * @requiredAssets img/tiles_game/TileA2
 * @requiredAssets img/tiles_game/TileA1
 * @requiredAssets
LISTA ZASOBOW - NIE ZMIENIAJ NIC NAD NIA
 */

(function() {

  var parameters = $plugins.filter(function(p) { return p.description.contains('<ChangeTileSize>'); })[0].parameters;
  var tileSize = parseInt(parameters['Rozmiar tile'] || 48);
  var tilesetsFolder = String(parameters['Folder z tilesetami'] || 'img/tilesets/');
  var parallaxesFolder = String(parameters['Folder z parallaksami'] || 'img/parallaxes/');

  ImageManager.loadTileset = function(filename, hue) {
    return this.loadBitmap(tilesetsFolder, filename, hue, false);
  };

  ImageManager.loadParallax = function(filename, hue) {
      return this.loadBitmap(parallaxesFolder, filename, hue, true);
  };

  Game_Map.prototype.tileWidth = function() {
    return tileSize;
  };

  Game_Map.prototype.tileHeight = function() {
    return tileSize;
  };

  Game_Vehicle.prototype.maxAltitude = function() {
    return tileSize;
  };

  var Tilemap_initialize = Tilemap.prototype.initialize;
  Tilemap.prototype.initialize = function() {
    Tilemap_initialize.call(this);
    this._tileWidth = tileSize;
    this._tileHeight = tileSize;
  };

  // Add image files to required assets list, to ensure they are included
  // in a deployed project
  var _changeTileSize_Scene_Boot_start = Scene_Boot.prototype.start;
  Scene_Boot.prototype.start = function() {
    _changeTileSize_Scene_Boot_start.call(this);

    if (!Utils.isOptionValid('test') || DataManager.isBattleTest() ||
      DataManager.isEventTest()) {
        return;
      }

    // Read in this script
    var fs = require('fs');
    var path = window.location.pathname.replace(/(\/www|)\/[^\/]*$/, '/');
    if (path.match(/^\/([A-Z]\:)/)) {
      path = path.slice(1);
    }
    path = decodeURIComponent(path);
    var scriptName = $plugins.filter(function(p) {
      return p.description.contains('<ChangeTileSize>') && p.status;
    })[0].name + '.js';
    var sourcePath = path + 'js/plugins/' + scriptName;
    var dataPath = path + 'data/';
    if (fs.existsSync(sourcePath)) {
      var source = fs.readFileSync(sourcePath, { encoding: 'utf8' });
      source = source.split('\r\n');
      var resListStart = source.indexOf('LISTA ZASOBOW - NIE ZMIENIAJ NIC POD NIA');
      var resListEnd = source.indexOf('LISTA ZASOBOW - NIE ZMIENIAJ NIC NAD NIA');
      if (resListStart === -1 || resListEnd === -1) {
        console.log('ERROR - Unable to create resource list for ' + scriptName);
        console.log('Please redownload the script and paste over your current version');
      } else {
        // remove previous list of resources
        source.splice(resListStart + 1, resListEnd - resListStart - 1);
        // build list of new resources
        var resTilesets = [];
        var resParallaxes = [];

        // get tilesets & parallaxes that are actually used
        $dataMapInfos.forEach(function(mapinfo) {
          if (mapinfo) {
            var map = JSON.parse(fs.readFileSync(dataPath + 'Map%1.json'.format(mapinfo.id.padZero(3))));

            if (!resTilesets.contains(map.tilesetId)) {
              resTilesets.push(map.tilesetId);
            }

            if (map.parallaxName !== '' && !resParallaxes.contains(map.parallaxName)) {
              resParallaxes.push(map.parallaxName);
            }

            // map events
            map.events.forEach(function (event) {
              if (event) {
                event.pages.forEach(function (page) {
                  if (page) {
                    getResources(page.list, resTilesets, resParallaxes);
                  }
                })
              }
            })
          }
        });

        // common events
        $dataCommonEvents.forEach(function(commonEvent) {
          if (commonEvent) {
            getResources(commonEvent.list, resTilesets, resParallaxes);
          }
        });

        // troop events
        $dataTroops.forEach(function (troop) {
          if (troop) {
            troop.pages.forEach(function (page) {
              if (page) {
                getResources(page.list, resTilesets, resParallaxes);
              }
            })
          }
        })

        // add resource list to script
        var inspos = resListStart + 1;

        resTilesetNames = [];
        resTilesets.forEach(function(tilesetId) {
          var tileset = $dataTilesets[tilesetId];
          if (tileset) {
            tileset.tilesetNames.forEach(function(tilesetName) {
              if (tilesetName !== '' && !resTilesetNames.contains(tilesetName)) {
                resTilesetNames.push(tilesetName);
              }
            })
          }
        });

        var tag = ' * @requiredAssets';
        source.splice(inspos, 0, tag);

        resTilesetNames.forEach(function(tilesetName) {
          tag = ' * @requiredAssets ' + tilesetsFolder + tilesetName;
          source.splice(inspos, 0, tag);
        });

        resParallaxes.forEach(function(parallaxName) {
          tag = ' * @requiredAssets ' + parallaxesFolder + parallaxName;
          source.splice(inspos, 0, tag);
        });

        // and save the file
        source = source.join('\r\n');
        fs.writeFileSync(sourcePath, source, { encoding: 'utf8' });
      }
    }
  };

  getResources = function(list, aryTilesets, aryParallaxes) {
    list.filter(function (cmd) {
      return cmd.code === 282 || cmd.code === 284;
    }).forEach(function(cmd) {
      if (cmd.code === 282 && !aryTilesets.contains(cmd.parameters[0])) {
        aryTilesets.push(cmd.parameters[0]);
      }
      if (cmd.code === 284 && !aryParallaxes.contains(cmd.parameters[0])) {
        aryParallaxes.push(cmd.parameters[0]);
      }
    });
  };
})();